/**
 * 
 */
package org.drools.base;

import java.util.HashSet;

import junit.framework.TestCase;

import org.drools.OuterClass;
import org.drools.OuterClass.InnerClass;
import org.drools.OuterClass.InnerClass.InnerInnerClass;
import org.drools.base.ClassTypeResolver;

/**
 * @author fburlet
 *
 */
public class ClassTypeResolverTest extends TestCase {

    public void testResolveObjectNotFromImport() throws Exception {
        final ClassTypeResolver resolver =  new ClassTypeResolver(new HashSet(), Thread.currentThread().getContextClassLoader());
        assertEquals( String.class,
                      resolver.resolveType( "String" ) );
        assertEquals( String.class,
                      resolver.resolveType( "java.lang.String" ) );
        try {
            assertEquals( OuterClass.class,
                          resolver.resolveType( "OuterClass" ) );
            fail( "Should raise a ClassNotFoundException" );
        } catch ( final ClassNotFoundException e ) {
            // success
        }
        assertEquals( OuterClass.class,
                      resolver.resolveType( "org.drools.OuterClass" ) );
        
        try {
            assertEquals( InnerClass.class,
                          resolver.resolveType( "InnerClass" ) );
            fail( "Should raise a ClassNotFoundException" );
        } catch ( final ClassNotFoundException e ) {
            // success
        }
        assertEquals( InnerClass.class,
                      resolver.resolveType( "org.drools.OuterClass.InnerClass" ) );
        
        try {
            assertEquals( InnerInnerClass.class,
                          resolver.resolveType( "InnerInnerClass" ) );
            fail( "Should raise a ClassNotFoundException" );
        } catch ( final ClassNotFoundException e ) {
            // success
        }
        assertEquals( InnerInnerClass.class,
                      resolver.resolveType( "org.drools.OuterClass.InnerClass.InnerInnerClass" ) );
    }

    public void testResolveObjectFromImport() throws Exception {
        final ClassTypeResolver resolver = new ClassTypeResolver( new HashSet(), Thread.currentThread().getContextClassLoader() );
        resolver.addImport( "org.drools.OuterClass" );
        resolver.addImport( "org.drools.OuterClass.InnerClass" );
        resolver.addImport( "org.drools.OuterClass.InnerClass.InnerInnerClass" ); 
        
        assertEquals( String.class,
                      resolver.resolveType( "String" ) );
        assertEquals( String.class,
                      resolver.resolveType( "java.lang.String" ) );
        assertEquals( OuterClass.class,
                      resolver.resolveType( "OuterClass" ) );
        assertEquals( OuterClass.class,
                      resolver.resolveType( "org.drools.OuterClass" ) );
        assertEquals( InnerClass.class,
                      resolver.resolveType( "org.drools.OuterClass.InnerClass" ) );
        assertEquals( InnerInnerClass.class,
                      resolver.resolveType( "org.drools.OuterClass.InnerClass.InnerInnerClass" ) );
    }
    
    public void testResolveObjectFromImportMultipleClassesDifferentPackages() throws Exception {
        final ClassTypeResolver resolver = new ClassTypeResolver( new HashSet(), Thread.currentThread().getContextClassLoader() );
        resolver.addImport( "org.drools.OuterClass" );
        resolver.addImport( "org.drools.OuterClass.InnerClass" );
        resolver.addImport( "org.drools.OuterClass.InnerClass.InnerInnerClass" );
        assertEquals( String.class,
                      resolver.resolveType( "String" ) );
        assertEquals( String.class,
                      resolver.resolveType( "java.lang.String" ) );
        assertEquals( OuterClass.class,
                      resolver.resolveType( "OuterClass" ) );
        assertEquals( OuterClass.class,
                      resolver.resolveType( "org.drools.OuterClass" ) );
        assertEquals( InnerClass.class,
                resolver.resolveType( "InnerClass" ) );
        assertEquals( InnerClass.class,
                resolver.resolveType( "org.drools.OuterClass.InnerClass" ) );
        assertEquals( InnerInnerClass.class,
                resolver.resolveType( "InnerInnerClass" ) );
        assertEquals( InnerInnerClass.class,
                resolver.resolveType( "org.drools.OuterClass.InnerClass.InnerInnerClass" ) );
    }

    public void testResolveArrayOfObjectsNotFromImport() throws Exception {
        final ClassTypeResolver resolver =  new ClassTypeResolver(new HashSet(), Thread.currentThread().getContextClassLoader());
        assertEquals( String[].class,
                      resolver.resolveType( "String[]" ) );
        assertEquals( String[].class,
                      resolver.resolveType( "java.lang.String[]" ) );
        try {
            assertEquals( OuterClass[].class,
                          resolver.resolveType( "OuterClass[]" ) );
            fail( "Should raise a ClassNotFoundException" );
        } catch ( final ClassNotFoundException e ) {
            // success
        }
        assertEquals( OuterClass[].class,
                      resolver.resolveType( "org.drools.OuterClass[]" ) );

        try {
            assertEquals( InnerClass[].class,
                          resolver.resolveType( "InnerClass[]" ) );
            fail( "Should raise a ClassNotFoundException" );
        } catch ( final ClassNotFoundException e ) {
            // success
        }
        assertEquals( InnerClass[].class,
                      resolver.resolveType( "org.drools.OuterClass.InnerClass[]" ) );
        

        try {
            assertEquals( InnerInnerClass[].class,
                          resolver.resolveType( "InnerInnerClass[]" ) );
            fail( "Should raise a ClassNotFoundException" );
        } catch ( final ClassNotFoundException e ) {
            // success
        }
        assertEquals( InnerInnerClass[].class,
                      resolver.resolveType( "org.drools.OuterClass.InnerClass.InnerInnerClass[]" ) );
    }

    public void testResolveArrayOfObjectsFromImport() throws Exception {
        final ClassTypeResolver resolver = new ClassTypeResolver( new HashSet(), Thread.currentThread().getContextClassLoader() );
        resolver.addImport( "org.drools.OuterClass" );
        resolver.addImport( "org.drools.OuterClass.InnerClass" );
        resolver.addImport( "org.drools.OuterClass.InnerClass.InnerInnerClass" );
        assertEquals( String[].class,
                      resolver.resolveType( "String[]" ) );
        assertEquals( String[].class,
                      resolver.resolveType( "java.lang.String[]" ) );
        assertEquals( OuterClass[].class,
                      resolver.resolveType( "OuterClass[]" ) );
        assertEquals( OuterClass[].class,
                      resolver.resolveType( "org.drools.OuterClass[]" ) );
        assertEquals( InnerClass[].class,
                resolver.resolveType( "OuterClass.InnerClass[]" ) );
        assertEquals( InnerClass[].class,
                resolver.resolveType( "org.drools.OuterClass.InnerClass[]" ) );
        assertEquals( InnerInnerClass[].class,
                resolver.resolveType( "OuterClass.InnerClass.InnerInnerClass[]" ) );
        assertEquals( InnerInnerClass[].class,
                resolver.resolveType( "InnerInnerClass[]" ) );
        assertEquals( InnerInnerClass[].class,
                resolver.resolveType( "org.drools.OuterClass.InnerClass.InnerInnerClass[]" ) );
    }

    public void testResolveMultidimensionnalArrayOfObjectsNotFromImport() throws Exception {
        final ClassTypeResolver resolver =  new ClassTypeResolver(new HashSet(), Thread.currentThread().getContextClassLoader());
        assertEquals( String[][].class,
                      resolver.resolveType( "String[][]" ) );
        assertEquals( String[][].class,
                      resolver.resolveType( "java.lang.String[][]" ) );
        try {
            assertEquals( OuterClass[][].class,
                          resolver.resolveType( "OuterClass[][]" ) );
            fail( "Should raise a ClassNotFoundException" );
        } catch ( final ClassNotFoundException e ) {
            // success
        }
        assertEquals( OuterClass[][].class,
                      resolver.resolveType( "org.drools.OuterClass[][]" ) );
        
        try {
            assertEquals( InnerClass[][].class,
                          resolver.resolveType( "InnerClass[][]" ) );
            fail( "Should raise a ClassNotFoundException" );
        } catch ( final ClassNotFoundException e ) {
            // success
        }
        assertEquals( InnerClass[][].class,
                      resolver.resolveType( "org.drools.OuterClass.InnerClass[][]" ) );
        
        try {
            assertEquals( InnerInnerClass[][].class,
                          resolver.resolveType( "InnerInnerClass[][]" ) );
            fail( "Should raise a ClassNotFoundException" );
        } catch ( final ClassNotFoundException e ) {
            // success
        }
        assertEquals( InnerInnerClass[][].class,
                      resolver.resolveType( "org.drools.OuterClass.InnerClass.InnerInnerClass[][]" ) );
    }

    public void testResolveMultidimensionnalArrayOfObjectsFromImport() throws Exception {
        final ClassTypeResolver resolver = new ClassTypeResolver( new HashSet(), Thread.currentThread().getContextClassLoader() );
        resolver.addImport( "org.drools.OuterClass" );
        resolver.addImport( "org.drools.OuterClass.InnerClass" );
        resolver.addImport( "org.drools.OuterClass.InnerClass.InnerInnerClass" );
        assertEquals( String[][].class,
                      resolver.resolveType( "String[][]" ) );
        assertEquals( String[][].class,
                      resolver.resolveType( "java.lang.String[][]" ) );
        assertEquals( OuterClass[][].class,
                      resolver.resolveType( "OuterClass[][]" ) );
        assertEquals( OuterClass[][].class,
                      resolver.resolveType( "org.drools.OuterClass[][]" ) );
        assertEquals( InnerClass[][].class,
                resolver.resolveType( "InnerClass[][]" ) );
        assertEquals( InnerClass[][].class,
                resolver.resolveType( "org.drools.OuterClass.InnerClass[][]" ) );
        assertEquals( InnerInnerClass[][].class,
                resolver.resolveType( "InnerInnerClass[][]" ) );
        assertEquals( InnerInnerClass[][].class,
                resolver.resolveType( "org.drools.OuterClass.InnerClass.InnerInnerClass[][]" ) );
    }
}

